<?php
/*
    "Contact Form to Database Extension" Copyright (C) 2011 Michael Simpson  (email : michael.d.simpson@gmail.com)

    This file is part of Contact Form to Database Extension.

    Contact Form to Database Extension is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Contact Form to Database Extension is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Contact Form to Database Extension.
    If not, see <http://www.gnu.org/licenses/>.

    Created by Kimiya Kitani (kitani@kitaney.jp) on October 18, 2012.
    Based on ExportToCsvUTF8.php. 

    For directly opening Microsoft Excel in Japanese, the Shift-JIS encoding must be used.
    However, Japanese encoding has very difficult issue because it is not single one.
    * For example:
         1. Shift-JIS is the standard encoding for Japanese Applications.
         2. EUC-JP is for older UNIX system.
         3. ISO-2022-JP (JIS) is the standard encoding for Japanese E-mail message.
         4. UTF-8 is the current popular encoding for latest devices or systems, such as Gmail, SmartPhone, and so on.

         Moreover, Shift-JIS includes special characters for Japanese.
 　      Then, UTF-8 only supports some characters among them because the capacity of UTF-8 is not enough.
     
         By coping with Japanese Microsoft Excel, 2 steps are needed.
           1. Fixed Shift-JIS encoding characters to UTF-8 because they may slip into DB.
           2. Convert UTF-8 to Shift-JIS supported Windows OS using mb_convert_encoding() function.

* mb_convert_encoding is include mb_string module(http://php.net/manual/en/book.mbstring.php). 
*/

require_once('ExportBase.php');
require_once('CFDBExport.php');

class ExportToCsvShiftJIS extends ExportBase implements CFDBExport {

    var $useBom = false;

    public function setUseBom($use) {
        $this->useBom = $use;
    }

    // Convert Shift-JIS (Standard Encoding for Japanese Applications) to UTF-8.
    public function japanese_convert_utf8_to_sjis ($str){
	$utf_escape_patterns_revert =array(
		// The code number of Japanese two-byte character "ー" is separated by Japanese encoding types. 
		'/\xE2\x80\x93/' => "\xE2\x88\x92",
	);
	
	$str =preg_replace(
		array_keys($utf_escape_patterns_revert),
		array_values($utf_escape_patterns_revert),
		$str);

	return $str;
    }

    public function export($formName, $options = null) {
        $this->setOptions($options);
        $this->setCommonOptions();

        // Security Check
        if (!$this->isAuthorized()) {
            $this->assertSecurityErrorMessage();
            return;
        }
	// If mb_convert_encoding function is not enabled (mb_string module is not installed), the converting is finished.
	if (!function_exists('mb_convert_encoding')){
	    return;
	}

        if ($this->options && is_array($this->options)) {
            if (isset($this->options['bom'])) {
                $this->useBom = $this->options['bom'] == 'true';
            }
        }

        // Headers
	// Change UTF-8 to Shift_JIS
        $this->echoHeaders(
            array('Content-Type: text/csv; charset=Shift_JIS',
                 "Content-Disposition: attachment; filename=\"$formName.csv\""));

        $this->echoCsv($formName);
    }

    public function echoCsv($formName) {
        if ($this->useBom) {
            // File encoding UTF-8 Byte Order Mark (BOM) http://wiki.sdn.sap.com/wiki/display/ABAP/Excel+files+-+CSV+format
            echo chr(239) . chr(187) . chr(191);
        }

        $eol = "\n";

        // Query DB for the data for that form
        $submitTimeKeyName = 'Submit_Time_Key';
        $this->setDataIterator($formName, $submitTimeKeyName);


        // Column Headers
        if (isset($this->options['header']) && $this->options['header'] != 'true') {
           // do not output column headers
        }
        else  {
            foreach ($this->dataIterator->displayColumns as $aCol) {
//                printf('"%s",', str_replace('"', '""', $aCol));
		// Convert UTF-8 to Shift-JIS supported Windows OS
                printf('"%s",', str_replace('"', '""', mb_convert_encoding($this->japanese_convert_utf8_to_sjis($aCol),"SJIS-win","utf-8")));
            }
            echo $eol;
        }

        // Rows
        $showFileUrlsInExport = $this->plugin->getOption('ShowFileUrlsInExport') == 'true';
        while ($this->dataIterator->nextRow()) {
            $fields_with_file = null;
            if ($showFileUrlsInExport &&
                    isset($this->dataIterator->row['fields_with_file']) &&
                    $this->dataIterator->row['fields_with_file'] != null) {
                $fields_with_file = explode(',', $this->dataIterator->row['fields_with_file']);
            }
            foreach ($this->dataIterator->displayColumns as $aCol) {
                $cell = isset($this->dataIterator->row[$aCol]) ? $this->dataIterator->row[$aCol] : '';
                if ($showFileUrlsInExport &&
                        $fields_with_file &&
                        $cell &&
                        in_array($aCol, $fields_with_file)) {
                    $cell = $this->plugin->getFileUrl($this->dataIterator->row[$submitTimeKeyName], $formName, $aCol);
                }
		// Convert UTF-8 to Shift-JIS supported Windows OS
//                printf('"%s",', str_replace('"', '""', $cell));
               printf('"%s",', str_replace('"', '""', mb_convert_encoding($this->japanese_convert_utf8_to_sjis($cell),"SJIS-win","utf-8")));

            }
            echo $eol;
        }
    }


}